mediump vec3 shift_col(lowp vec3 RGB, mediump vec3 shift)
{
    // lowp only guarantees 8 bits of precision, and we are using 3 digit decimal values...
    const mediump vec3 r00      = vec3( 0.299,  0.299,  0.299);
    const mediump vec3 r01      = vec3( 0.701, -0.299, -0.300);
    const mediump vec3 r02      = vec3( 0.168, -0.328,  1.250);
    
    const mediump vec3 r10      = vec3( 0.587,  0.587,  0.587);
    const mediump vec3 r11      = vec3(-0.587,  0.413, -0.588);
    const mediump vec3 r12      = vec3( 0.330,  0.035, -1.050);
    
    const mediump vec3 r20      = vec3( 0.114,  0.114,  0.114);
    const mediump vec3 r21      = vec3(-0.587, -0.114,  0.886);
    const mediump vec3 r22      = vec3(-0.497,  0.292, -0.203);
    
    return ((r00 * shift.z + r01 * shift.x + r02 * shift.y) * RGB.r +
            (r10 * shift.z + r11 * shift.x + r12 * shift.y) * RGB.g +
            (r20 * shift.z + r21 * shift.x + r22 * shift.y) * RGB.b);
}

// First half of the Overlay effect
// Should be used with <0 srcColor> blend mode.

// Map Dest pixel to:
//    src < 0.5 => 2*src*dest  (output = 2*Src)
//    src > 0.5 => dest        (output = 1)

lowp vec4 overlay_multiply_col(lowp vec4 col)
{
    lowp vec4 fval = step(0.5, col);
    lowp vec4 col2 = col * 2.0;
    return col2 - fval * col2 + fval;
}

// Second half of the Overlay effect
// Should be used with <1, invSrcColor> blend mode.

// Map Dest pixel to:
//    src < 0.5 => dest                               (output = 0)
//    src > 0.5 => 1 - 2 * (1 - src)*(1 - dest)       (output = 2 * src - 1)

lowp vec4 overlay_screen_col(lowp vec4 col)
{
    lowp vec4 fval = step(0.5, col);
    lowp vec4 col2 = col * 2.0 - vec4(1.0);
    return fval * col2;
}

// Should be used with <destColor, srcColor> blend mode.
lowp vec4 color_dodge_col(lowp vec4 col)
{
    return vec4(1.0)/(vec4(2.0) - col);   // Maps [0..1] to [0.5..1].
    // Using <src, dest>, 0 => preserve the dest channel, 1 => double the dest channel.
}


highp vec2 compute_frag_uv(highp vec2 vert_uv)
{
#ifdef UV_SCROLL
    // Very slow way to do this (dependent texture read, per-pixel logic, etc.)
    // But realistically, we have to support atlased textures and we are a long way from being GPU bound.
    // (Also, this let's art use NPOT textures.)
    return mod(vert_uv - drift_shift_scale.xy, drift_shift_scale.zw) + drift_shift_scale.xy;
#else
    return vert_uv;
#endif
}
