


mat4 get_model_matrix()
{
#ifdef INSTANCED_TRANSFORMS
    mat4 mm = model_matrices[gl_InstanceIDEXT];

    mm[0][3] = 0.0;
    mm[1][3] = 0.0;
    mm[2][3] = 0.0;
    mm[3][3] = 1.0;
    return mm;
#else
    return model_matrix;
#endif
}

#define GET_INSTANCE_COLOR_DEFINED
vec4 get_instance_color()
{
#ifdef INSTANCED_TRANSFORMS
    mat4 mm = model_matrices[gl_InstanceIDEXT];
    return vec4(mm[0][3], mm[1][3], mm[2][3], mm[3][3]);
#else
    return vec4(1.0, 1.0, 1.0, 1.0);
#endif
}

mat4 get_mvp()
{
#ifdef INSTANCED_TRANSFORMS
    //return camera * model_matrices[gl_InstanceIDEXT]; // This fails due to diver bug on adreno 320 and 330
    mat4 mm = get_model_matrix();
    return camera * mm;
#else
    return mvp;
#endif
}

#ifndef kHasCustomAttributes
#ifdef kHasAnimation
mat4 ix_animation_mat4_transpose()
{
    // Each animation matrix is passed through as 3 vec4's representing the
    // first three rows of the matrix
    mat4 result;
    int k = 3*int(bones.x);
    result[0] = animation[k] * weights.x;
    result[1] = animation[k + 1] * weights.x;
    result[2] = animation[k + 2] * weights.x;
    result[3] = vec4(0.0, 0.0, 0.0, 1.0);

    k = 3*int(bones.y);
    result[0] += animation[k] * weights.y;
    result[1] += animation[k + 1] * weights.y;
    result[2] += animation[k + 2] * weights.y;

    k = 3*int(bones.z);
    result[0] += animation[k] * weights.z;
    result[1] += animation[k + 1] * weights.z;
    result[2] += animation[k + 2] * weights.z;

    k = 3*int(bones.w);
    result[0] += animation[k] * weights.w;
    result[1] += animation[k + 1] * weights.w;
    result[2] += animation[k + 2] * weights.w;

    // Since OpenGL uses column major order this is the transpose of the original animation matrix
    return result;
}


vec4 ix_animation_transform4(vec4 tpos)
{
    return tpos * ix_animation_mat4_transpose();
}

vec4 ix_transform()
{
    #ifdef kHasNormalizedPosition
        vec4 tpos = vec4(pos * mesh_scale, 1.0);
    #else
        vec4 tpos = vec4(pos, 1.0);
    #endif

        vec4 new_position = ix_animation_transform4(tpos);

        return mvp * vec4(new_position.xyz, 1.0);
}

vec3 ix_pos_transform(vec3 posIn)
{
#ifdef kHasNormalizedPosition
    vec4 tpos = vec4(posIn * mesh_scale, 1.0);
#else
    vec4 tpos = vec4(posIn, 1.0);
#endif

#ifdef kHasAnimation
    vec4 new_position = ix_animation_transform4(tpos);
    return new_position.xyz;
#else
    return tpos.xyz;
#endif

}

#else // does not kHasAnimation

vec4 ix_transform()
{
    #ifdef kHasNormalizedPosition
        vec3 tpos = vec3(pos * mesh_scale);
    #else
        vec3 tpos = vec3(pos);
    #endif
        return get_mvp() * vec4(tpos, 1.0);
}

vec3 ix_pos_transform(vec3 posIn)
{
#ifdef kHasNormalizedPosition
    return vec3(posIn * mesh_scale);
#else
    return posIn;
#endif
}

#endif

vec3 ix_pos_transform()
{
    return ix_pos_transform(pos);
}

#ifdef kHasNormals

vec3 ix_nml_transform()
{
    #ifdef kHasAnimation

    vec4 tnml = vec4(nml, 0.0);
    vec4 new_nml = ix_animation_transform4(tnml);

    return new_nml.xyz;
    #else

    return nml;

    #endif
}

#else

vec3 ix_nml_transform()
{
    return vec3(0.0, 0.0, 1.0);
}

#endif

// #define OPTIMIZE_MATRIX_MUL 1

#ifdef kHasTangents

mat3 mat3_emu(mat4 m4)
{
    return mat3(
        m4[0][0], m4[0][1], m4[0][2],
        m4[1][0], m4[1][1], m4[1][2],
        m4[2][0], m4[2][1], m4[2][2]);
}

mat3 mat3_vec(vec3 a, vec3 b, vec3 c)
{
    return mat3(
        a.x, b.x, c.x,
        a.y, b.y, c.y,
        a.z, b.z, c.z);

}

void ix_tangent_space(in vec3 in_to_light, in vec3 in_eye_dir, out vec3 out_to_light, out vec3 out_half)
{

    mat3 basis_mat;

#ifdef kHasAnimation
    mat3 sum_mat = mat3_emu(ix_animation_mat4_transpose());

    vec3 tnml = nml * sum_mat;
    vec3 ttgt = tgt * sum_mat;

    basis_mat = mat3_vec(ttgt, cross(tnml, ttgt), tnml);

#else


    #ifndef kShaderGeomType_Binormal
        // If no tangents defined in the vertex, create one
        vec3 bin = cross(nml, tgt);
    #endif

    basis_mat = mat3_vec(tgt, bin, nml);

#endif

    out_to_light = normalize(basis_mat * in_to_light);
    out_half = out_to_light - normalize(basis_mat * in_eye_dir);

}

vec3 ix_tan_transform()
{

#ifdef kHasAnimation
    vec4 tan = vec4(tgt, 0.0);
    vec4 new_tan = ix_animation_transform4(tan);

    return new_tan.xyz;
#else
    return tgt;
#endif

}

vec3 ix_bin_transform()
{
#ifdef kHasBinormals
    #ifdef kHasAnimation
        vec4 bnm = vec4(bin, 0.0);
        vec4 new_bin = ix_animation_transform4(bnm);

        return new_bin.xyz;
    #else
        return bin;
    #endif
#else
    return cross (nml, tgt);
#endif
}

#else       // !kHasTangents

void ix_tangent_space(in vec3 in_to_light, in vec3 in_eye_dir, out vec3 out_to_light, out vec3 out_half)
{
    // Pass through - do nothing
}

vec3 ix_tan_transform()
{
    return vec3(1, 0, 0);
}
#endif


vec4 ix_pos_transform_anim_return_vec4(vec3 posIn, mat4 anim)
{
    #ifdef kHasNormalizedPosition
        vec4 tpos = vec4(posIn * mesh_scale, 1.0);
    #else
        vec4 tpos = vec4(posIn, 1.0);
    #endif

    #ifdef kHasAnimation
        return tpos * anim;
    #else
        return tpos;
    #endif
}


vec3 ix_pos_transform(vec3 posIn, mat4 anim)
{
    return ix_pos_transform_anim_return_vec4(pos, anim).xyz;
}


vec3 ix_pos_transform(mat4 anim)
{
    return ix_pos_transform(pos, anim);
}

vec3 ix_nml_transform(mat4 anim)
{
    #ifdef kHasNormals
        #ifdef kHasAnimation
            vec4 tnml = vec4(nml, 0.0);
            vec4 new_nml = tnml * anim;
            return new_nml.xyz;
        #else
            return nml;
        #endif
    #else // !kHasNormals
        return vec3(0.0, 0.0, 1.0);
    #endif
}

vec3 ix_tan_transform(mat4 anim)
{
#ifdef kHasTangents
    #ifdef kHasAnimation
        vec4 tan = vec4(tgt, 0.0);
        vec4 new_tan = tan * anim;
        return new_tan.xyz;
    #else
        return tgt;
    #endif
#else // !kHasTangents
    return vec3(1, 0, 0);
#endif // End ifdef kHasTangents
}

vec3 ix_bin_transform(mat4 anim)
{
#ifdef kHasBinormals
    #ifdef kHasAnimation
        vec4 bnm = vec4(bin, 0.0);
        vec4 new_bin = bnm * anim;
        return new_bin.xyz;
    #else
        return bin;
    #endif
#else
    #if defined kHasNormals && defined kHasTangents
        return cross (nml, tgt);
    #else
        return vec3(0, 1, 0);
    #endif
#endif
}

#endif

mediump vec2 compute_vertex_uv(vec2 uv)
{
#ifdef UV_SCROLL
    return uv + uv_offset;
#else
    return uv;
#endif
}

vec3 getProbeLightContribution(in mediump vec3 iNml)
{
#ifdef PROBE_LIGHTING

    mediump vec4 vNormal = vec4(iNml.x, iNml.y, iNml.z, 1.0);
    mediump vec3 x1, x2, x3;

    // Linear + constant polynomial terms
    x1.r = dot(SHcAr,vNormal);
    x1.g = dot(SHcAg,vNormal);
    x1.b = dot(SHcAb,vNormal);


    // 4 of the quadratic polynomials
    mediump vec4 vB = vNormal.xyzz * vNormal.yzzx;
    x2.r = dot(SHcBr,vB);
    x2.g = dot(SHcBg,vB);
    x2.b = dot(SHcBb,vB);


    // Final quadratic polynomial
    mediump float vC = vNormal.x*vNormal.x - vNormal.y*vNormal.y;
    x3 = SHcC.rgb * vC;
    return x1+x2+x3;
#else
    return vec3(0.0,0.0,0.0);
#endif

}
