uniform highp vec3 shadowdir;

void main()
{
    highp vec3 oPos = ix_pos_transform();
    vec4 tnml = vec4(nml, 0.0);     // fix Android hack
                                    // This needs to be after the first pos
                                    // access to work on the older kindles
    mat4 model_mat = get_model_matrix();

#ifdef WORLD_SPACE_FLAT_SHADOWS
    vec4 wPos = model_mat * vec4(oPos.x, oPos.y, oPos.z, 1.0);
  #ifdef WORLD_SPACE_SHADOWS_Z_VERTICAL
    wPos.xyz -= shadowdir*wPos.z/shadowdir.z;
  #else
    wPos.xyz -= shadowdir*wPos.y/shadowdir.y;
  #endif

    highp vec4 clipPos = camera * wPos;

#else

    // transform light direction to model space
    highp mat3 model_mat_inv3 = mat3(model_mat[0].xyz, model_mat[1].xyz, model_mat[2].xyz);
    highp vec3 mLightDir = normalize(shadowdir * model_mat_inv3);

    // project shadow onto "ground" in local space
    oPos -= mLightDir*oPos.y/mLightDir.y;
    highp vec4 clipPos = get_mvp() * vec4(oPos, 1.);

#endif

#ifdef FORCE_DEPTH_TO_FAR_PLANE
    // perform our own perspecive divide to make sure
    // we can make the depth match exactly for each shadow pixel
    if (clipPos.w > 0.)
        clipPos = vec4(clipPos.xy/clipPos.w, .9999, 1.);
#endif

    gl_Position = clipPos;
}

