#ifdef GL_ES
precision highp float;
#endif


varying vec2 vtex0;
varying vec2 vtex1;
varying highp vec3 world_position;
varying highp vec3 light_contribution;

uniform sampler2D diffuse_map;

uniform highp float pbr_exposure;

#ifdef LIGHTMAP
uniform sampler2D light_map;
uniform highp vec2 pbr_lm_exposure;
#endif

#ifdef FOG
varying float vfog;
uniform vec2 fogdist;
uniform vec3 fogcolor;
uniform vec2 fogheight;
#endif

#ifdef DEMATERIALIZING
uniform float elapsed_time;
uniform float dematerialization_time;
#endif

void main()
{
#ifdef DEMATERIALIZING
    highp float t1 = 2.0 * (dematerialization_time - elapsed_time);
    highp float t2 = 0.40*(0.5-mod(t1 * 5.0, 1.0)) + t1;
    highp vec3 coords = vec3(mod(world_position.x * 10.0, 1.0), mod(world_position.y * 10.0, 1.0), mod(world_position.z * 10.0, 1.0));
    if(t2 < mod((3.2345 * coords.x + 0.367)*( 3.9824 * coords.y  + 0.2345) * (3.678 * coords.z + 0.7892), 1.0))
        discard;
#endif

    highp vec4 diffuseColor = texture2D (diffuse_map, vtex0);
    highp vec3 diffuseColorLin = pow (diffuseColor.rgb, vec3 (2.2)); // srgb to linear

#ifdef LIGHTMAP
    highp vec3 lightmapContrib = pow (texture2D (light_map, vtex1).rgb, vec3 (2.2)) * pbr_lm_exposure.x + vec3(pbr_lm_exposure.y);
    highp vec3 finalColor = (light_contribution + lightmapContrib) * diffuseColorLin;
#else
    highp vec3 finalColor = light_contribution * diffuseColorLin;
#endif

    // exposure and tonemapping
    vec3 x = max(vec3(0.0), pbr_exposure * finalColor.xyz - vec3(0.004));
    finalColor.xyz = (x * (6.2 * x + vec3(0.5))) / (x * (6.2 * x + vec3(1.7)) + vec3(0.06));

#ifdef FOG
    float fog = clamp ((vfog - fogdist.x) / (fogdist.y - fogdist.x), 0.0, 1.0);
    fog *= 1.0 - clamp((world_position.y - fogheight.x) / (fogheight.y - fogheight.x), 0.0, 1.0);
    finalColor = mix (finalColor, fogcolor, fog);
#endif

    gl_FragColor = vec4 (finalColor, diffuseColor.a) * global_color;
}
