#ifdef GL_ES
precision highp float;
#endif

uniform mat4 transform;
varying highp vec3 world_position;
varying highp vec3 world_normal;
varying highp vec3 light_contribution;

varying vec2 vtex0;
varying vec2 vtex1;
attribute vec2 tex0;

#ifdef HAS_TEXT_COORDS2
attribute vec2 tex1;
#endif

#ifdef FOG
varying float vfog;
#endif

uniform lowp float pbr_active_lights;
uniform highp mat4 pbr_light_data[kMaxPBRLights];
uniform highp vec4 pbr_ambient;

void main()
{
    highp vec4 object_position = vec4(ix_pos_transform(), 1.0);
    world_position = (transform * object_position).xyz;
    gl_Position = get_mvp() * object_position;

    mat3 model_matrix33 = mat3(transform[0].xyz, transform[1].xyz, transform[2].xyz);
    world_normal = normalize (model_matrix33 * ix_nml_transform());

    vtex0 = tex0;
    vtex1 = tex0;
#ifdef HAS_TEXT_COORDS2
    vtex1 = tex1;
#endif

#ifdef FOG
    vfog = ix_transform().z;
#endif
    light_contribution = vec3(0.0);
    for (int i = 0; i < int(pbr_active_lights); ++i)
    {
        highp mat4 light_data               = pbr_light_data[i];
        highp vec4 light_pos_intensity      = light_data[0];
        highp vec4 light_dir_type           = light_data[1];
        highp vec4 light_diff_inner_cone    = light_data[2];
        highp vec4 light_spec_outer_cone    = light_data[3];
        float non_directional = 1.0 + min (0.0, light_dir_type.w);
        highp vec3 LD = -light_dir_type.xyz + non_directional * ((light_pos_intensity.xyz - world_position) + light_dir_type.xyz);
        highp float d = length (LD);
        highp float I = light_pos_intensity.w / max(0.01, d * d);
        highp vec3 L = normalize (LD);
        highp float cone = clamp((dot (L, -normalize (light_dir_type.xyz)) - light_spec_outer_cone.w) / (light_diff_inner_cone.w - light_spec_outer_cone.w), 0.0, 1.0);
        I *= 1.0 + max(0.0, light_dir_type.w) * (cone - 1.0);
        I *= max(0.0, dot(world_normal, L));

        light_contribution += I * light_diff_inner_cone.xyz;
    }
    light_contribution += pbr_ambient.rgb * pbr_ambient.a;
}
