
// mass animation vertex shader

attribute float pos;
varying vec2 vtex0;
varying vec3 wNormal;
varying vec3 wPosition;
varying vec3 wTangent;
varying vec3 wBitangent;
varying vec4 vcol;
const float TWO_TO_THE_MINUS_10 = 1.0 / 1024.0; // 155
const int kNumEntities = 50;
uniform highp vec3 light_dir[kMaxLights];
uniform highp vec4 entityPositionRotation[kNumEntities]; // xyz => pos, w => cos and sin packed with 10 bits
uniform highp vec4 entityColorPosV[kNumEntities];      // xyz => color, w => position v index
uniform highp vec2 vertsIVerts;                    // vertex count, 1 / vertex count
uniform highp vec4 uvShiftScaleVValueNormalVOffset;     // Row values in animation texture for uv, pos shift, and pos scale.  w=>v offset from pos row to normal row.
uniform highp vec4 meshScale;                           // w is world scale
uniform highp vec3 meshShift;
uniform highp vec4 cameraTilt;

uniform sampler2D animSource;

void main()
{
#ifdef INSTANCED
    float vertId = pos;
    int entityId = gl_InstanceIDEXT;
#else
    float vertId = mod(pos, vertsIVerts.x);
    int entityId = int((pos - vertId) / vertsIVerts.x + 0.5);
#endif
    float scaledVertId = (vertId + 0.5) * vertsIVerts.y;
    highp vec4 ePR = entityPositionRotation[entityId];
    vec4 eCV = entityColorPosV[entityId];

    vec2 posSourceCoords = vec2(scaledVertId, eCV.w);
    vec2 norSourceCoords = vec2(scaledVertId, eCV.w + uvShiftScaleVValueNormalVOffset.w);
    vec2 tanSourceCoords = vec2(scaledVertId, eCV.w + 2.0*uvShiftScaleVValueNormalVOffset.w);

    vec4 texPosition = texture2DLod(animSource, posSourceCoords, 0.0);
    wPosition = texPosition.xyz;

    wPosition.xyz *= meshScale.xyz;                                         // Map from unit cube to world scale units
    wPosition.xyz += meshShift;

    wPosition.xyz *= meshScale.w;                                           // Scale to world dimensions

    vec4 nor = texture2DLod(animSource, norSourceCoords, 0.0);
    vec4 tan = texture2DLod(animSource, tanSourceCoords, 0.0);
    wNormal = (2.0 * nor.xyz) - 1.0;
    wTangent = (2.0 * tan.xyz) - 1.0;

    vtex0 = vec2(texPosition.w, nor.w);

    //unpack rotation
    vec2 rot;
    rot.x = floor(ePR.w) * TWO_TO_THE_MINUS_10;
    rot.y = 2.0*(ePR.w - floor(ePR.w) - 0.5);

    // Rotate position, normal, and tangent
    wNormal.xz = vec2(rot.x * wNormal.x - rot.y * wNormal.z, rot.y * wNormal.x + rot.x * wNormal.z);
    wTangent.xz = vec2(rot.x * wTangent.x - rot.y * wTangent.z, rot.y * wTangent.x + rot.x * wTangent.z);
    wPosition.xz = vec2(rot.x * wPosition.x - rot.y * wPosition.z, rot.y * wPosition.x + rot.x * wPosition.z);
    wBitangent = cross(wNormal, wTangent); // No reason for normalization, normal and tangent should be orthogonal unit vectors

    wPosition.y += ePR.z;

    //Camera tilt
    wPosition.y = cameraTilt.x * wPosition.y + cameraTilt.y * wPosition.z;
    wPosition.z = cameraTilt.z * wPosition.y + cameraTilt.w * wPosition.z;

    wPosition.xy += ePR.xy;

    vcol = vec4(eCV.rgb, 1.0);

    gl_Position = (mvp * vec4(wPosition,1.0)); // model matrix should be identity so this is really "vp"
}
