varying highp vec2 vtex0;
varying highp vec2 vtex1;

uniform sampler2D pattern;
uniform sampler2D texture1;
uniform sampler2D texture2;
uniform sampler2D texture3;
uniform sampler2D lightmap;
uniform highp float tile;

uniform highp float lmap_mult;
uniform highp float lmap_add;
uniform highp float lmap_gamma;

highp vec3 fast_vec3pow(highp vec3 a, highp vec3 b)
{
    return a/((vec3(1.,1.,1.) - b)*a + b);
}

void main()
{
    highp vec4 patternColor = texture2D(pattern, vtex0);
    highp vec2 tileUv = vtex0 * tile;
    tileUv.x = tileUv.x - floor(tileUv.x);
    tileUv.y = tileUv.y - floor(tileUv.y);

    highp vec4 texture1Color = texture2D(texture1, tileUv);
    highp vec4 texture2Color = texture2D(texture2, tileUv);
    highp vec4 texture3Color = texture2D(texture3, tileUv);
    highp vec4 diffuseColor = texture1Color * patternColor.r +
                              texture2Color * patternColor.g +
                              texture3Color * patternColor.b;

    highp vec3 lmap = fast_vec3pow (texture2D (lightmap, vtex1).rgb, vec3 (lmap_gamma));
    lmap += lmap_add;
    lmap *= lmap_mult;
    lmap *= fast_vec3pow (diffuseColor.rgb, vec3 (lmap_gamma));

    highp vec4 outcolor;
    outcolor.rgb = fast_vec3pow (lmap, vec3 (1.0/lmap_gamma));
    outcolor.a = diffuseColor.a;

    gl_FragColor = outcolor;
}
