varying vec2 vtex0;
varying highp vec4 light_color;

attribute vec2 tex0;

//uniform vec3 light_pos[kMaxLights];
uniform highp vec3 light_dir[kMaxLights];

uniform vec4 light_diffuse[kMaxLights];
uniform vec4 light_ambient[kMaxLights];

uniform highp float scroll;
uniform highp float horizontalFreq;
uniform highp float horizontalScale;
uniform highp float verticalFreq;
uniform highp float verticalScale;

float sinf(float x)
{
    x *= 0.159155;
    x -= floor(x);
    float xx = x * x;
    float y = -6.87897;
    y = y * xx + 33.7755;
    y = y * xx - 72.5257;
    y = y * xx + 80.5874;
    y = y * xx - 41.2408;
    y = y * xx + 6.28077;
    return x * y;
}


// saturate
float sat(float src)
{
    return clamp(src, 0., 1.);
}

void main()
{
    vtex0 = tex0;


    mat4 model = get_model_matrix();
    vec3 oPos = ix_pos_transform(); // object space position
    oPos.x += sinf(scroll * horizontalFreq) * horizontalScale * oPos.y;
    oPos.y += sinf(scroll * verticalFreq) * verticalScale * (oPos.x * oPos.x + oPos.z + oPos.z) / 10.0;
    vec4 wPos = model * vec4(oPos, 1.); // world space position


    // must normalize because transform can contain a scale
    mat3 transform3 = mat3(model[0].xyz, model[1].xyz, model[2].xyz);
    vec3 oNormal = ix_nml_transform(); // Object space normal
    vec3 wNormal = normalize(transform3 * oNormal); // World space normal


    // Match the ligting calculations done in normalmapwsl_frag
    // This is taking light0 as the sky, light1 as the sun and
    // adding in a small contribution for light bouncing off the ground
    float dpSky = -dot(light_dir[0], wNormal);
    float dpSun = -dot(light_dir[1], wNormal);
    vec3 ground = light_ambient[0].rgb*sat(-dpSky * 0.8 + 0.2);
    vec3 sky = light_diffuse[0].rgb*sat(dpSky * 0.4 + 0.6);
    vec3 sun = light_diffuse[1].rgb*sat(dpSun);

    light_color.rgb = ground + sky + sun;
    light_color.a = 1.0;

    #ifdef GET_INSTANCE_COLOR_DEFINED
        light_color *= get_instance_color();
    #endif

    gl_Position = camera * wPos;
}
