varying vec2 vtex0;
varying highp vec4 light_color;

attribute vec2 tex0;

//uniform vec3 light_pos[kMaxLights];
uniform highp vec3 light_dir[kMaxLights];

uniform vec4 light_diffuse[kMaxLights];
uniform vec4 light_ambient[kMaxLights];


// saturate
float sat(float src)
{
    return clamp(src, 0., 1.);
}


void main()
{
    vtex0 = tex0;
   

    mat4 model = get_model_matrix();
    vec3 oPos = ix_pos_transform(); // object space position
    vec4 wPos = model * vec4(oPos, 1.); // world space position


    // must normalize because transform can contain a scale
    mat3 transform3 = mat3(model[0].xyz, model[1].xyz, model[2].xyz);
    vec3 oNormal = ix_nml_transform(); // Object space normal
    vec3 wNormal = normalize(transform3 * oNormal); // World space normal

    // Match the ligting calculations done in normalmapwsl_frag
    // This is taking light0 as the sky, light1 as the sun and
    // adding in a small contribution for light bouncing off the ground
    float dpSky = -dot(light_dir[0], wNormal);
    float dpSun = -dot(light_dir[1], wNormal);
    vec3 ground = light_ambient[0].rgb*sat(-dpSky * 0.8 + 0.2);
    vec3 sky = light_diffuse[0].rgb*sat(dpSky * 0.4 + 0.6);
    vec3 sun = light_diffuse[1].rgb*sat(dpSun);

    light_color.rgb = ground + sky + sun;
    light_color.a = 1.0;

    #ifdef GET_INSTANCE_COLOR_DEFINED
        light_color *= get_instance_color();
    #endif

    gl_Position = camera * wPos;
}
