varying highp vec2 vtex0;
varying highp vec2 vtex1;
varying highp vec2 vtex2;
varying highp vec2 vtex3;
varying highp vec2 vtex4;
varying highp vec3 vnml;
varying highp vec3 vpos;
varying highp vec4 clipPos;
varying highp vec3 eye_pos_frac;

uniform highp vec3 eye_dir_w;

uniform sampler2D diffuse;
uniform sampler2D normal;
uniform sampler2D sky;
uniform sampler2D foam;
uniform sampler2D lightmap;

uniform highp float lmap_mult;
uniform highp float lmap_add;
uniform highp float lmap_gamma;

uniform highp float shininess;
uniform highp float scroll;
uniform highp float zoom_min;
uniform highp float zoom_max_inv;

uniform highp float discard_thresh;
uniform highp float water_scale;
uniform highp float sun_spot_size;
uniform highp float wave_sharpness;

uniform highp vec4 light_diffuse[kMaxLights];
uniform highp vec4 light_ambient[kMaxLights];
uniform highp vec4 light_specular[kMaxLights];

mediump float fast_pow(highp float a, highp float b)
{
    return a/((1. - b)*a + b);
}

void main()
{
    highp vec4 tex_src0 = texture2D(diffuse, mod(vtex0, 1.));

    if (tex_src0.a < discard_thresh)
        discard;

    lowp float water = float((tex_src0.b > tex_src0.r && (tex_src0.b >= tex_src0.g || tex_src0.b > 0.1)));

    // Counter the effect of black bleeding into samples by dividing out alpha.
    // Prevents dark lines from forming at water tile boundaries.  EN-1100.
    tex_src0.xyz /= vec3(tex_src0.a, tex_src0.a, tex_src0.a);
    tex_src0.a = 1.0;

    // This offset is tied to the camera FOV, wonder if we can make that official
    highp vec3 vto_light = eye_pos_frac - (vpos + vec3(-100, 0.0, -100));
    vto_light.y *= .2;

    vto_light.x *= sun_spot_size;
    vto_light.z *= sun_spot_size;
    vto_light = normalize(vto_light);

    //highp float dist = 1.0;
    highp float dist = distance(eye_pos_frac, vpos);
    dist = clamp((dist - zoom_min) * zoom_max_inv, 0.0, 1.0);

    highp float a = tex_src0.a;

    highp vec3 nml = texture2D(normal, vtex2).xyz;
    highp vec3 nml2 = texture2D(normal, vtex3).xyz;
    highp vec3 nml3 = texture2D(normal, vtex4).xyz;

    nml = normalize(((2.0/3.0) * (nml + nml2 + nml3) - 1.0) * vec3(wave_sharpness, wave_sharpness, wave_sharpness));  // factor out?

    highp float ndotl = dot(nml, vto_light);
    highp float diff = clamp(ndotl, 0.0, 1.0);
    highp float spec = clamp(fast_pow(ndotl, shininess), 0.0, 1.0);
    highp vec4 light_color = (dist * spec * light_specular[0]) + (diff * light_diffuse[0]) + light_ambient[0];

    mediump vec3 localColor = tex_src0.xyz * 1. + 0.2 * water *  light_color.xyz;

    highp vec2 reflection = (clipPos.xy / clipPos.w) / 2. + 0.5;
    highp vec3 reflectionColor = texture2D(sky, reflection - scroll * 0.05).xyz * 0.15;

    highp vec4 watercolor = (vec4(localColor, wave_sharpness) + vec4(reflectionColor, 0.3)) * global_color * vcol; 

    highp vec3 lmap = pow (texture2D (lightmap, vtex1).rgb, vec3 (lmap_gamma));
    lmap += lmap_add;
    lmap *= lmap_mult;
    lmap *= pow (watercolor.rgb, vec3 (lmap_gamma));

    highp vec4 outcolor;
    outcolor.rgb = pow (lmap, vec3 (1.0/lmap_gamma));
    outcolor.a = watercolor.a;

    gl_FragColor = outcolor;

}
