
#ifdef GL_ES
precision mediump float;
#endif

varying highp vec2 vtex0;
varying highp vec3 wNormal;
varying highp vec3 wTangent;
varying highp vec3 wBitangent;
varying highp vec3 wPosition;

uniform sampler2D diffuse;
uniform sampler2D normalmap;
uniform sampler2D specmap;

uniform highp vec4 light_diffuse[kMaxLights];
uniform highp vec4 light_ambient[kMaxLights];
uniform highp vec4 light_specular[kMaxLights];
uniform highp vec3 light_dir[kMaxLights];
uniform highp vec3 light_pos[kMaxLights];

uniform lowp float weight;

uniform highp vec3 eye_pos_w;

uniform highp float shininess;

float specPow(float a, float b)
{
    return a/((1. - b)*a + b);
}

// saturate
float sat(float src)
{
    return clamp(src, 0., 1.);
}

void main()
{
    vec4 specData = texture2D(specmap, vtex0);
    vec3 tNormal = normalize((2.0 * texture2D(normalmap, vtex0).rgb) - 1.0);
    vec3 n = tNormal.x * wTangent + tNormal.y * wBitangent + tNormal.z * wNormal;
    n = normalize(n);
    
    vec3 u = normalize(wPosition - eye_pos_w);
    highp vec4 diffuseColor = texture2D(diffuse, vtex0);
    diffuseColor.rgb *= diffuseColor.rgb;
    
    float dpSky = -dot(light_dir[0], n);
    float dpSun = -dot(light_dir[1], n);
    vec3 ground = light_ambient[0].rgb*sat(-dpSky * 0.8 + 0.2);
    vec3 sky = light_diffuse[0].rgb*sat(dpSky * 0.4 + 0.6);

    float z = light_ambient[1].z - mod(wPosition.z + light_ambient[1].z - light_pos[1].z, light_ambient[1].w);
    highp vec3 v = vec3(light_pos[1].xy - wPosition.xy, z);  // vector to (nearest) light
    highp float d = length(v);
    v /= d;

    float spotLightCone = sat(-dot(v, light_dir[1].xyz)*light_ambient[1].x + light_ambient[1].y);
    float dpSpot = sat(-dot(light_dir[1].xyz, n));
    vec3 spot = light_diffuse[1].rgb*light_diffuse[1].a/(d*d + light_diffuse[1].a) * spotLightCone;
    
    vec3 r = u - 2.*n*dot(n, u);
    
    float smoothness = specData.g;

    float fresnel = clamp(specPow(sat(1.1 + dot(n, u)), 3.) + .1, 0., 1.);
    float specIntensity = specData.r*8.0*(smoothness+.125) * fresnel;
    float smoothPower = smoothness*80.+1.;
    
    float skyCos = dot(r, light_dir[0]);
    float skySpec = specPow(sat(-skyCos*.7+.3), smoothPower)*specIntensity*sat(dpSky*8.);
    float groundSpec = specPow(sat(skyCos*0.8+0.2), smoothPower)*specIntensity*sat(-dpSky*8.);
    float sunSpec = specPow(sat(-dot(r, light_dir[1].xyz)), smoothPower)*specIntensity*sat(dpSpot*8.);

    
    vec3 final_color = diffuseColor.rgb*(ground + sky + spot * dpSpot);
    final_color += skySpec*light_diffuse[0].rgb;
    final_color += groundSpec*light_ambient[0].rgb;
    final_color += spot*sunSpec;

    final_color = sqrt(final_color);

    gl_FragColor = vec4(.5, 1.2, 2., 0.25)*(weight*fresnel) + vec4(final_color,1.0)*(1.0 - weight);
}
