
#ifdef GL_ES
precision mediump float;
#endif

varying highp vec3 wPosition;
varying highp vec3 wNormal;

#ifdef HAS_TEXT_COORDS
varying highp vec2 vtex0;
varying highp vec3 wTangent;
varying highp vec3 wBitangent;
#endif

uniform sampler2D diffuse;
uniform sampler2D normalmap;
uniform sampler2D specmap;

uniform highp vec4 light_diffuse[kMaxLights];
uniform highp vec4 light_ambient[kMaxLights];
uniform highp vec4 light_specular[kMaxLights];
uniform highp vec3 light_dir[kMaxLights];

uniform highp vec3 eye_pos_w;

uniform highp float shininess;

float specPow(float a, float b)
{
    return a/((1. - b)*a + b);
}

// saturate
float sat(float src)
{
    return clamp(src, 0., 1.);
}

void main()
{
#ifdef HAS_TEXT_COORDS
    vec4 specData = texture2D(specmap, vtex0);
    vec3 tNormal = normalize((2.0 * texture2D(normalmap, vtex0).rgb) - 1.0);
    vec3 n = tNormal.x * wTangent + tNormal.y * wBitangent + tNormal.z * wNormal;
    n = normalize(n);
    
    vec3 u = normalize(wPosition - eye_pos_w);
    highp vec4 diffuseColor = texture2D(diffuse, vtex0);
    diffuseColor.rgb *= global_color.rgb*(1. - specData.b) + specData.b;
    diffuseColor.rgb *= diffuseColor.rgb;
#else
    vec3 n = wNormal;
    highp vec4 diffuseColor = vec4(global_color.rgb, 1.0);
#endif
   
    float dpSky = -dot(light_dir[0], n);
    float dpSun = -dot(light_dir[1], n);
    vec3 ground = light_ambient[0].rgb*sat(-dpSky * 0.8 + 0.2);
    vec3 sky = light_diffuse[0].rgb*sat(dpSky * 0.4 + 0.6);
    vec3 sun = light_diffuse[1].rgb*sat(dpSun);
 
#ifdef HAS_TEXT_COORDS
    vec3 r = u - 2.*n*dot(n, u);
    
    float smoothness = specData.g;
    
    float specIntensity = specData.r*8.0*(smoothness+.125);
    float smoothPower = smoothness*80.+1.;
    
    float skyCos = dot(r, light_dir[0]);
    float skySpec = specPow(sat(-skyCos*.7+.3), smoothPower)*specIntensity*sat(dpSky*8.);
    float groundSpec = specPow(sat(skyCos*0.8+0.2), smoothPower)*specIntensity*sat(-dpSky*8.);
    float sunSpec = specPow(sat(-dot(r, light_dir[1])), smoothPower)*specIntensity*sat(dpSun*8.);
    
    float fresnel = clamp(specPow(sat(1.1 + dot(n, u)), 3.) + .1, 0., 1.);
    skySpec *= fresnel;
    groundSpec *= fresnel;
    sunSpec *= fresnel;
#endif
    vec3 final_color = diffuseColor.rgb*(ground + sky + sun);

#ifdef HAS_TEXT_COORDS
#ifdef EMISSIVE
    float emissive = specData.b;

    final_color = final_color * (1. - emissive) + diffuseColor.rgb*emissive;
#endif

    final_color += skySpec*light_diffuse[0].rgb;
    final_color += groundSpec*light_ambient[0].rgb;
    final_color += sunSpec*light_diffuse[1].rgb;
#else
#endif

    final_color = sqrt(final_color);
    
    gl_FragColor = vec4(final_color, diffuseColor.a * global_color.a)*vcol;
}
