// Basically normalmapwsl_vert with some extra values calculate


varying vec2 vtex0;
varying vec3 wNormal;
varying vec3 wTangent;
varying vec3 wBitangent;
varying vec3 wPosition;

varying vec3 portalPos; // Point in coordinates relative to the portal (scaled)
varying vec3 portalEyeDir; // Eye direction to point on surface relative to the portal (scaled)

uniform highp vec3 eye_pos_w;
uniform highp vec3 eye_dir_w;

uniform highp vec4 portal_window_xy; // minX, maxX, minY, maxY
uniform highp float portal_window_z;

attribute vec2 tex0;

uniform mat4 transform;
uniform mat4 parent_matrix_inv;


void main()
{
    // compute gl_Position first to force z to match depth_only.prog
    highp vec3 oPos = ix_pos_transform();
    gl_Position = (mvp * vec4(oPos, 1.)); //ix_transform();

    vec3 objectSpaceNormal = ix_nml_transform();
    vec3 objectSpaceTangent = ix_tan_transform();

    // must normalize because transform can contain a scale
    mat3 transform3 = mat3(transform[0].xyz, transform[1].xyz, transform[2].xyz);
    wNormal = normalize(transform3 * objectSpaceNormal);
    wTangent = transform3 * objectSpaceTangent;
    wBitangent = normalize(cross(wNormal, wTangent));
    wTangent = cross(wBitangent, wNormal);
    wPosition = (transform * vec4(oPos, 1.)).xyz;
    vtex0 = tex0;
    vcol = vec4(1.0,1.0,1.0,1.0); //Needed to share fragment shader with massanimation.prog
    

    // Now portal specific calculations
    highp vec4 ppos = parent_matrix_inv * vec4(wPosition, 1.);
    vec3 portalCenter = vec3(0.5*(portal_window_xy.x + portal_window_xy.y), 0.5*(portal_window_xy.z + portal_window_xy.w), portal_window_z);
    vec2 portalSize = vec2(portal_window_xy.y - portal_window_xy.x,  portal_window_xy.w - portal_window_xy.z);
    vec3 portalInvScale = vec3(2.0/portalSize.x, 2.0/portalSize.y, 1.0);
    portalPos = portalInvScale*(ppos.xyz - portalCenter); // Normalized Model position relative to portal center
    vec3 viewDir  =   projection[2][3] != 0.0  ? (wPosition - eye_pos_w) : eye_dir_w; // Perspective or Orthorgraphic
    portalEyeDir =  portalInvScale*( (parent_matrix_inv * vec4(viewDir, 1.0)).xyz ) ; // Perspective
}




