varying vec2 vtex0;
#ifdef QUADMATERIAL
varying vec4 vtex1;
#endif
varying vec3 wNormal;
varying vec3 wTangent;
varying vec3 wBitangent;
varying vec3 wPosition;
#ifdef PORTAL
varying vec3 portalPos; // Point in coordinates relative to the portal (scaled)
varying vec3 portalEyeDir; // Eye direction to point on surface relative to the portal (scaled)
uniform highp vec4 portal_window_xy; // minX, maxX, minY, maxY
uniform highp float portal_window_z;
uniform mat4 parent_matrix_inv;
#endif

uniform highp vec3 eye_pos_w;
uniform highp vec3 eye_dir_w;

#if defined(VERTEX_LIGHTING) | defined(RIM_LIGHT)
uniform highp vec4 light_diffuse[kMaxLights];
uniform highp vec4 light_ambient[kMaxLights];
uniform highp vec3 light_dir[kMaxLights];
#endif

#ifdef VERTEX_LIGHTING
varying highp vec3 irradiance1;
varying highp vec3 irradiance2;
varying highp vec3 irradiance3;
#endif

#ifdef RIM_LIGHT
varying highp vec3 light1Dir;
#endif

#ifdef MORPH
attribute vec4 tex0;
uniform sampler2D vdmap;
uniform float vdmapScale;
uniform vec4 morphWeight;
#else
attribute vec2 tex0;
#endif

uniform mat4 transform;

#ifdef ANIMATION_AO
uniform sampler2D aomap;
uniform vec4 aomapSize;
#endif

#ifdef SHADOW_MAP
uniform mat4 ShadowTransform;
uniform float ShadowBias;
varying vec4 shadowmapUV;
#endif

#ifdef ANIMATED_TEXCOORDS
uniform float rows;
uniform float cols;
uniform float fps;
#endif

vec3 sat(vec3 src)
{
    return clamp(src, 0., 1.);
}

void main()
{
    vec3 vpos = pos;
#ifdef MORPH
    highp vec3 vdN = vec3(128./255., 128./255., 128./255.);
    highp vec3 vd = (texture2DLod(vdmap, tex0.zw + vec2(.5, 0.), 0.).xyz - vdN)*morphWeight.x;
    vd += (texture2DLod(vdmap, tex0.zw + vec2(0., .5), 0.).xyz - vdN)*morphWeight.y;
    vd += (texture2DLod(vdmap, tex0.zw + vec2(.5, .5), 0.).xyz - vdN)*morphWeight.z;
    vd = (vd*2. - 1.) * vdmapScale;
    vpos += vd;
#endif

    highp vec3 oPos = ix_pos_transform(vpos);
    // must normalize because transform can contain a scale
    mat3 transform3 = mat3(transform[0].xyz, transform[1].xyz, transform[2].xyz);
    wNormal = normalize(transform3 * ix_nml_transform());
    wTangent = transform3 * ix_tan_transform();
    wBitangent = normalize(cross(wNormal, wTangent));
    wTangent = cross(wBitangent, wNormal);
    wPosition = (transform * vec4(oPos, 1.)).xyz;
    vtex0 = tex0.xy;

#ifdef ANIMATED_TEXCOORDS
    float frame = mod (floor (elapsed_time * fps), rows * cols);
    float row   = floor (frame / cols);
    float col   = frame - (cols * row); // modulo using previous floored divide (row).
    
    vec2 delta_st = 1.0 / vec2 (cols, rows);
    
    vtex0.st *= delta_st.st;
    vtex0.st += (delta_st * vec2 (col, row));
#endif
    
    gl_Position = (get_mvp() * vec4(oPos, 1.));

#ifdef QUADMATERIAL
    vtex1 = vec4(tex0, tex0 + 1.)*0.5;
#endif

#ifdef SHADOW_MAP
    vec4 depthmapPos = ShadowTransform * vec4(wPosition, 1.);
    depthmapPos = (depthmapPos + depthmapPos.w)*0.5;
    depthmapPos.z -= ShadowBias;
    shadowmapUV = depthmapPos;
#endif

#if defined(PORTAL) | defined(RIM_LIGHT)
    vec3 viewDir  =   projection[2][3] != 0.0  ? (wPosition - eye_pos_w) : eye_dir_w; // Perspective or Orthorgraphic
#endif

#ifdef PORTAL
    // Now portal specific calculations
    highp vec4 ppos = parent_matrix_inv * vec4(wPosition, 1.);
    vec3 portalCenter = vec3(0.5*(portal_window_xy.x + portal_window_xy.y), 0.5*(portal_window_xy.z + portal_window_xy.w), portal_window_z);
    vec2 portalSize = vec2(portal_window_xy.y - portal_window_xy.x,  portal_window_xy.w - portal_window_xy.z);
    vec3 portalInvScale = vec3(2.0/portalSize.x, 2.0/portalSize.y, 1.0);
    portalPos = portalInvScale*(ppos.xyz - portalCenter); // Normalized Model position relative to portal center
    portalEyeDir =  portalInvScale*( (parent_matrix_inv * vec4(viewDir, 1.0)).xyz ) ; // Perspective
#endif

#ifdef RIM_LIGHT
    light1Dir = light_dir[1];
    if (light_diffuse[1].a > 0.)
    {
        vec3 bb = normalize(cross(viewDir, vec3(0., 1., 0.)));
        light1Dir = normalize(-viewDir - cross(bb, viewDir)*light_diffuse[1].a);
    }
#endif

#ifdef VERTEX_LIGHTING
    vec3 nn = wNormal * 0.949;
    vec3 tt = wTangent * 0.316;
    vec3 bb = wBitangent * 0.316 *0.866;

    vec3 lv1 = nn + tt;
    vec3 lv2 = nn+bb-tt*0.5;
    vec3 lv3 = nn-bb-tt*0.5;

    float spread = 0.5;

    vec3 skylightDir = light_dir[0].rgb;
    vec3 skylightColor = light_diffuse[0].rgb;
    vec3 bouncelightColor = light_ambient[0].rgb;

    vec3 v = -skylightDir;
    vec3 w = sat(vec3(dot(v, lv1), dot(v, lv2), dot(v, lv3))*(1.-spread) + spread);
    irradiance1 = skylightColor * w.x;
    irradiance2 = skylightColor * w.y;
    irradiance3 = skylightColor * w.z;

    spread = 0.25;
    v = skylightDir;
    w = sat(vec3(dot(v, lv1), dot(v, lv2), dot(v, lv3))*(1.-spread) + spread);
    irradiance1 += bouncelightColor * w.x;
    irradiance2 += bouncelightColor * w.y;
    irradiance3 += bouncelightColor * w.z;
    
#ifdef PROBE_LIGHTING
    irradiance1 += getProbeLightContribution(lv1);
    irradiance2 += getProbeLightContribution(lv2);
    irradiance3 += getProbeLightContribution(lv3);
#endif

#ifdef ANIMATION_AO
    vec4 patchIndex = ix_vertex_color();

    vec2 uv = floor(patchIndex.zw*255. + 1.) * 0.03125;
    vec2 mUV = 1. - uv;

    highp vec2 aoUV = (floor(patchIndex.xy*255. + .1)*3. + 1. + uv*.5)*aomapSize.zw;
    highp vec4 aoUV2 = vec4(aoUV, aoUV + aomapSize.zw*.5);

    vec3 vertexAO = texture2DLod(aomap, aoUV2.xy, 0.).rgb*mUV.x*mUV.y
    + texture2DLod(aomap, aoUV2.zy, 0.).rgb*uv.x*mUV.y
    + texture2DLod(aomap, aoUV2.zw, 0.).rgb*uv.x*uv.y
    + texture2DLod(aomap, aoUV2.xw, 0.).rgb*mUV.x*uv.y;

    vcol.rgb = vertexAO.rbg;

    irradiance1 *= vertexAO.r;
    irradiance2 *= vertexAO.b;
    irradiance3 *= vertexAO.g;
#endif
#endif
}
